@@ -30,7 +30,7 @@ def titlecase(part_of_speech, word, is_first_word, is_last_word, name):
                 where = "at the end of a "
             message = Report.misformatted(what="lowercase", where=where + name)
 
-            fix = word.slice(word.start_pos, word.start_pos + 1, True)
+            fix = word.slice(end=word.start_pos + 1, after_inner=True)
             fix.replace(str(fix).swapcase())
 
             return message, fix
@@ -47,12 +47,39 @@ def titlecase(part_of_speech, word, is_first_word, is_last_word, name):
         fix = None
         if (not tag or (tag[0] != "preposition" and
                 (tag[-1] != "article" or word_str != "A"))):
-            fix = word.slice(word.start_pos, word.start_pos + 1, True)
+            fix = word.slice(end=word.start_pos + 1, after_inner=True)
             fix.replace(str(fix).swapcase())
 
         return message, fix
 
 
+def abbr_caps_pre(_):
+    char_catalog = CharCatalog()
+    re_lib = dict()
+    re_lib["noupper"] = re.compile(r"[^" + char_catalog.unicode_set("A-Z", 0, 7) + r"]")
+    return {"re_lib": re_lib}
+
+
+def abbr_caps(toolname, document, reports, re_lib):
+    """Check if uppercase letters of an abbreviation match those of the explanation."""
+    for node in rst_walker.iter_node(document.body, "role"):
+        if rst_walker.is_of(node, "*", "abbr"):
+            if not node.id:
+                reports.append(
+                    Report('F', toolname, node.code, "abbreviation without an explanation"))
+                continue
+
+            uppers_head = re.sub(re_lib["noupper"], "", str(node.head))
+            uppers_id = re.sub(re_lib["noupper"], "", str(node.id))
+            if uppers_head != uppers_id:
+                reports.append(
+                    Report('W', toolname, node.code,
+                           "abbreviation uppercase mismatch: {0} - {1}"
+                           .format(uppers_head, uppers_id)))
+
+    return reports
+
+
 def admonition_title(toolname, document, reports):
     """Case of admonition titles."""
     segmenter = Segmenter()
@@ -72,9 +99,10 @@ def admonition_title(toolname, document, reports):
                         if word_str[0].islower():
                             word_low += 1
                 if word_all > 1 and word_low/ word_all >= threshold:
-                    message = "admonition caption titlecase: {:4.0%}".format(word_low/ word_all)
-                    output = node.head.code.copy().clear(True)
-                    reports.append(Report('W', toolname, output, message, node.head.code))
+                    reports.append(
+                        Report('W', toolname, node.head.code.copy().clear(True),
+                               "admonition caption titlecase: {:4.0%}"
+                               .format(word_low/ word_all), node.head.code))
 
     return reports
 
@@ -99,7 +127,6 @@ def heading_caps_pre(_):
 
 def heading_caps(toolname, document, reports, re_lib):
     """Check the heading title capitalization."""
-
     segmenter = Segmenter()
     part_of_speech = PartofSpeech()
     instr_pos = {
@@ -107,7 +134,7 @@ def heading_caps(toolname, document, reports, re_lib):
     }
     instr_neg = {
         "role": {
-            "kbd": "*", "menuselection": "*", "math": "*"
+            "kbd": "*", "menuselection": "*", "math": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -124,8 +151,8 @@ def heading_caps(toolname, document, reports, re_lib):
                 if buf:
                     if message_repl := titlecase(part_of_speech, buf, is_first_word,
                                                  False, "heading"):
-                        reports.append(Report('W', toolname, buf, message_repl[0], node.name.code,
-                                              message_repl[1]))
+                        reports.append(Report('W', toolname, buf, message_repl[0],
+                                              node.name.code, message_repl[1]))
                     if is_faq:
                         break
                     is_first_word = False
@@ -136,15 +163,15 @@ def heading_caps(toolname, document, reports, re_lib):
                 is_last_word = bool(part.parent_node.next is None)
                 if message_repl := titlecase(part_of_speech, buf, is_first_word,
                                              is_last_word, "heading"):
-                    reports.append(Report('W', toolname, buf, message_repl[0], node.name.code,
-                                          message_repl[1]))
+                    reports.append(Report('W', toolname, buf, message_repl[0],
+                                          node.name.code, message_repl[1]))
                 is_first_word = False
 
             part_str = str(part.code)
             for pattern, message in re_lib.values():
                 for m in re.finditer(pattern, part_str):
-                    output = part.code.slice_match_obj(m, 0, True)
-                    reports.append(Report('W', toolname, output, message, node.name.code))
+                    reports.append(Report('W', toolname, part.code.slice_match(m, 0, True),
+                                          message, node.name.code))
 
     return reports
 
@@ -161,14 +188,14 @@ def pos_case(toolname, document, reports):
     instr_neg = {
         "dir": {
             "figure": ["head"],
-            "admonition": ["head"], "hint": ["head"], "important": ["head"],
-            "note": ["head"], "tip": ["head"], "warning": ["head"], "rubric": ["head"],
+            "hint": ["head"], "important": ["head"], "note": ["head"],
+            "tip": ["head"], "warning": ["head"], "rubric": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*", "toctree": "*",
-            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*"
+            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "def": {"*": ["head"]},
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*", "comment": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": "*", "emphasis": "*", "int-target": "*", "hyperlink": "*",
         "literal": "*", "standalone": "*"
     }
@@ -182,7 +209,7 @@ def pos_case(toolname, document, reports):
                     not rst_walker.is_of(node_par.parent_node, "document")):
                 continue
 
-        for sen, is_open in segmenter.iter_sentence(part.code, output_openess=True):
+        for sen, stop in segmenter.iter_sentence(part.code):
             is_first_word = not was_open
             for word in segmenter.iter_word(sen):
                 if is_first_word:
@@ -195,10 +222,11 @@ def pos_case(toolname, document, reports):
 
                 tag = part_of_speech.tag(word_str.lower())
                 if len(tag) != 0 and tag[0] not in {"noun", "abbreviation", "adjective", "verb"}:
-                    message = Report.misformatted(what="uppercase " + tag[0])
-                    reports.append(Report('W', toolname, word, message, sen))
+                    reports.append(
+                        Report('W', toolname, word,
+                               Report.misformatted(what="uppercase " + tag[0]), sen))
 
-            was_open = is_open
+            was_open = bool(stop is None)
 
         # paragraph end
         if not part.parent_node.next:
@@ -220,12 +248,12 @@ def proper_noun_pre(_):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*", "toctree": "*",
-            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*"
+            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*", "comment": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "math": "*"
+            "kbd": "*", "math": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -237,7 +265,7 @@ def proper_noun_pre(_):
 
         first = True
         for part in rst_walker.iter_nodeparts_instr(document.body, instr_pos, instr_neg):
-            for sen in segmenter.iter_sentence(part.code):
+            for sen, _ in segmenter.iter_sentence(part.code):
                 for word in segmenter.iter_word(sen):
                     if first:
                         first = False
@@ -284,9 +312,9 @@ def proper_noun(toolname, document, reports, data, config):
                                                 config["instr_neg"]):
         for word in segmenter.iter_word(part.code):
             if entry := data.find(str(word)):
-                message = "proper noun: {:4.0%}".format(entry["ratio"])
-                line = Report.getline_punc(document.code, word, 50, 30)
-                reports.append(Report('W', toolname, word, message, line))
+                reports.append(
+                    Report('W', toolname, word, "proper noun: {:4.0%}".format(entry["ratio"]))
+                    .set_line_punc(document.code, 50, 30))
 
     return reports
 
@@ -324,7 +352,6 @@ def start_case_pre(_):
 
 def start_case(toolname, document, reports, re_lib):
     """Check case at the start of paragraphs, sentences and parenthesis."""
-
     instr_pos = {
         "field": {"*": ["name", "body"]},
         "*": {"*": ["head", "body"]}
@@ -332,7 +359,7 @@ def start_case(toolname, document, reports, re_lib):
     instr_neg = {
         "dir": {
             "figure": ["head"], "toctree": "*", "include": "*", "index": "*", "highlight": "*",
-            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*"
+            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
         "doctest": "*",
@@ -347,9 +374,9 @@ def start_case(toolname, document, reports, re_lib):
                     part.parent_node.parent_node.parent_node.node_name == "text"):
 
                 if re.match(start_re, str(part.code)):
-                    output = part.code.copy().clear(True)
-                    line = Report.getline_offset(document.body.code, output, 100, True)
-                    reports.append(Report('W', toolname, output, re_lib["lowerpara"][1], line))
+                    reports.append(
+                        Report('W', toolname, part.code.copy().clear(True), re_lib["lowerpara"][1])
+                        .set_line_offset(document.body.code, 100, True))
 
                 was_empty = bool(len(part.code) == 0)
 
@@ -360,9 +387,9 @@ def start_case(toolname, document, reports, re_lib):
                     continue
                 pattern = value[0]
                 for m in re.finditer(pattern, part_str):
-                    output = part.code.slice_match_obj(m, 0, True)
-                    line = Report.getline_offset(document.body.code, output, 100, True)
-                    reports.append(Report('W', toolname, output, value[1], line))
+                    reports.append(
+                        Report('W', toolname, part.code.slice_match(m, 0, True), value[1])
+                        .set_line_offset(document.body.code, 100, True))
 
     return reports
 
@@ -410,7 +437,11 @@ def typ_case_pre(_):
                         pattern_str.extend(words[index + 1:])
                         term.append(r"\b" + " ".join(pattern_str))
 
-            terms.append([term, head])
+            for _, head_rec in terms:
+                if head_rec == head:
+                    break
+            else:
+                terms.append([term, head])
 
             break
 
@@ -422,7 +453,7 @@ def typ_case_pre(_):
         ("Node", "compositing/types/", ["groups"]),
         ("Node", "render/shader_nodes/", ["osl"]),
         ("Node", "editors/texture_node/types/", []),
-        ("Node", "modeling/modifiers/nodes/", []),
+        ("Node", "modeling/geometry_nodes/", ["group"]),
         ("Strip", "video_editing/sequencer/strips/", [])
     )
     terms = []
@@ -448,21 +479,30 @@ def typ_case_pre(_):
                 terms = typ_titles(document, kind, terms)
 
     args = dict()
-    args["config"] = listsearch.parse_flags("")
-    args["data"] = listsearch.compile_terms(terms, {"flags": args["config"]})
-
+    args["config"] = {}
+    args["data"] = listsearch.compile_terms(terms, {"flags": listsearch.parse_flags("")})
     return args
 
 
 def typ_case(toolname, document, reports, data, config):
-    return listsearch.search(toolname, document, reports, data, config)
+    return listsearch.search_char(toolname, document, reports, data, config)
 
 
 def ui_case(toolname, document, reports):
-    """Check the capitalization in definition list terms."""
-    def check_words(part, code, is_first_word, part_name, line, do_fix=False):
+    """Check the capitalization of UI terms in reference lists."""
+    def check_words(part, code, is_first_word, part_name, line, is_field=False):
         buf = None
+        do_fix = False
         ui_terms = {"menu", "context", "node", "tab", "panel", "region", "editor", "editors"}
+        attr_terms = {
+            "align", "alt", "class", "encoding", "end-before", "end-line", "figclass", "figwidth",
+            "header-rows", "height", "literal", "name", "number-lines", "parser",
+            "scale", "start-after", "start-line", "stub-columns", "subtitle", "tab-width",
+            "target", "width", "widths",
+        }
+        if is_field and str(code) not in attr_terms:
+            do_fix = True
+
         for word in segmenter.iter_word(code):
             if buf:
                 if str(buf) in ui_terms:
@@ -496,9 +536,9 @@ def ui_case(toolname, document, reports):
     }
     instr_neg = {
         "role": {
-            "kbd": "*", "menuselection": "*", "math": "*"
+            "kbd": "*", "menuselection": "*", "math": "*", "default": "*"
         },
-        "standalone": "*", "literal": "*", "substitution": "*"
+        "literal": "*", "standalone": "*", "substitution": "*"
     }
     icon_re = re.compile(r"\([^\)]*?\)\s*\Z")
     arrow_re = re.compile(r"\s*\-+>\s*\b")
@@ -551,6 +591,7 @@ def ui_case(toolname, document, reports):
 
 
 OPS = (
+    ("abbr-caps", abbr_caps, abbr_caps_pre),
     ("admonition-title", admonition_title, None),
     ("heading-caps", heading_caps, heading_caps_pre),
     ("pos-case", pos_case, None),
