@@ -16,15 +16,13 @@ from monostyle.rst_parser.core import RSTParser
 
 def highlight_pre(toolname):
     config = dict()
-    config.update(monostyle_io.get_override(__file__, toolname, "threshold_min", 0.6))
-    config.update(monostyle_io.get_override(__file__, toolname, "threshold_severe", 1.0))
+    config.update(monostyle_io.get_override(__file__, toolname, "thresholds", (0.6, 1.0)))
     return {"config": config}
 
 
 def highlight(toolname, document, reports, config):
     """Overuse of inline markup."""
-    threshold_min = config["threshold_min"]
-    threshold_severe = config["threshold_severe"]
+    thresholds = config["thresholds"]
     average_ref = 14
     blank_line = 80
 
@@ -41,12 +39,13 @@ def highlight(toolname, document, reports, config):
 
         if text_chars > 100 or is_final:
             score_final = (score_global / counter) * (-(1 / (counter + (1 / 1.125))) + 1.125)
-            if score_final > threshold_min:
-                output = score_cur["node"].code.copy().clear(True)
-                message = Report.quantity(what="highlight overuse",
-                                          how=str(counter) + " times")
-                reports.append(Report('I' if score_final < threshold_severe else 'W', toolname,
-                                      output, message, score_cur["node"].parent_node.code))
+            if score_final > thresholds[0]:
+                reports.append(
+                    Report(Report.map_severity(thresholds, score_final), toolname,
+                           score_cur["node"].code.copy().clear(True),
+                           Report.quantity(what="highlight overuse",
+                                           how=str(counter) + " times"),
+                           score_cur["node"].parent_node.code))
 
             score_global = 0
             counter = 0
@@ -66,6 +65,7 @@ def highlight(toolname, document, reports, config):
     score_global = 0
     counter = 0
     line_trim_re = re.compile(r"^\s*(\S.*?)\s*$", re.MULTILINE)
+    space_re = re.compile(r"\S\s")
     text_chars = 0
     for node_parent in rst_walker.iter_node(document.body, {"text", "dir", "sect"}, False):
         if node_parent.node_name != "text":
@@ -127,6 +127,19 @@ def highlight(toolname, document, reports, config):
             score_cur = score_next
             text_chars = 0
 
+            # individual whitespace-based word count
+            if node.node_name in {"emphasis", "strong"}:
+                space_counter = sum(1 for _ in re.finditer(space_re, str(node.body.code)))
+                threshold_len = (4, 6) if node.node_name == "emphasis" else (3, 5)
+                if space_counter >= threshold_len[0]:
+                    reports.append(
+                        Report(Report.map_severity(threshold_len, space_counter),
+                               toolname, node.body.code.copy().clear(True),
+                               Report.existing(what="long",
+                                               where=node.node_name +
+                                                     ": {0} words".format(space_counter + 1)),
+                               node.body.code))
+
     if score_cur:
         reports, score_global, counter = evaluate(reports, score_global, counter,
                                                   score_cur, text_chars, True)
@@ -136,7 +149,6 @@ def highlight(toolname, document, reports, config):
 
 def heading_level(toolname, document, reports):
     """Heading hierarchy defined by the under/overline char."""
-
     level_chars = ('%', '#', '*', '=', '-', '^', '"', "'")
     levels = {level_char: index for index, level_char in enumerate(level_chars)}
     title_count = 0
@@ -187,15 +199,15 @@ def heading_level(toolname, document, reports):
             level_prev = level_cur
 
         if message is not None:
-            output = node.name_end.code.copy().replace_fill(heading_char)
-            reports.append(Report(severity, toolname, output, message))
+            reports.append(
+                Report(severity, toolname,
+                       node.name_end.code.copy().replace_fill(heading_char), message))
 
     return reports
 
 
 def indention(toolname, document, reports):
     """Check RST code line indention."""
-
     default_indent = 3
     field_ind_hanging = 3
     field_ind_fix = 15
@@ -223,9 +235,11 @@ def indention(toolname, document, reports):
                 if block_line.start_lincol[1] < ind_aim and offset != ind_aim:
                     with_what += (" or {:+} chars (col: {})"
                                   .format(offset - block_line.start_lincol[1], offset + 1))
-                message = Report.substitution(what="wrong indent", with_what=with_what)
-                output = block_line.clear(True)
-                reports.append(Report('E', toolname, output, message, block_line_full))
+
+                reports.append(
+                    Report('E', toolname, block_line.clear(True),
+                           Report.substitution(what="wrong indent", with_what=with_what),
+                           block_line_full))
 
         return reports
 
@@ -248,17 +262,18 @@ def indention(toolname, document, reports):
                 if (ind_trg_next is not None and
                         node.indent.code.end_lincol[1] != ind_trg_next.code.end_lincol[1]):
                     # further intended
-                    message = Report.existing(what="target", where="not on same indent level")
-                    reports.append(Report('W', toolname, node.id.code, message))
+                    reports.append(
+                        Report('W', toolname, node.id.code,
+                               Report.existing(what="target", where="not on same indent level")))
             else:
                 # unintended
-                message = Report.missing(what="target", where="on same indent level")
-                reports.append(Report('W', toolname, node.id.code, message))
+                reports.append(
+                    Report('W', toolname, node.id.code,
+                           Report.missing(what="target", where="on same indent level")))
 
         # limit: groups of aligned fields, todo favor fix on refbox
         elif node.node_name == "field-list":
-            # if (rst_walker.is_of(node.parent_node, "dir", "admonition") and
-                  # str(rst_walker.get_attr(node.parent_node, "class")).strip() == "refbox"):
+            # if rst_walker.is_of(node.parent_node, "dir", "reference"):
 
             inds = []
             for node_field in rst_walker.iter_node(node.body, "field"):
@@ -342,20 +357,18 @@ def indention(toolname, document, reports):
                         ind_aim_block = ind_aim_first
 
                 if ind_first != ind_aim_first:
-                    message = Report.substitution(what="field wrong alignment",
-                                                  with_what="{:+} chars".format(
-                                                  ind_aim_first - ind_first))
-
-                    output = node_field.name_end.code.copy().clear(False)
-                    reports.append(Report('W', toolname, output, message, node_field.name.code))
+                    reports.append(
+                        Report('W', toolname, node_field.name_end.code.copy().clear(False),
+                               Report.substitution(what="field wrong alignment",
+                                   with_what="{:+} chars".format(ind_aim_first - ind_first)),
+                               node_field.name.code))
 
                 if ind_block is not None and ind_block != ind_aim_block:
-                    message = Report.substitution(what="field wrong indent",
-                                                  with_what="{:+} chars".format(
-                                                  ind_aim_block - ind_block))
-
-                    output = node_field.name_end.code.copy().clear(False)
-                    reports.append(Report('W', toolname, output, message, node_field.name.code))
+                    reports.append(
+                        Report('W', toolname, node_field.name_end.code.copy().clear(False),
+                               Report.substitution(what="field wrong indent",
+                                   with_what="{:+} chars".format(ind_aim_block - ind_block)),
+                               node_field.name.code))
 
         else:
             if (node.node_name.endswith("-list") or
@@ -399,12 +412,13 @@ def indention(toolname, document, reports):
                     elif markup_space_m := re.search(markup_space_re, str(part.code)):
                         if (len(markup_space_m.group(1)) != 1 and
                                 not rst_walker.is_of(part, "substdef", "*", "id_end")):
-                            with_what = "{:+} chars".format(1 - len(markup_space_m.group(1)))
-                            message = Report.substitution(what="wrong markup spacing",
-                                                          with_what=with_what)
-                            line = (node.child_nodes[2].code if len(node.child_nodes) > 1
-                                    else node.code)
-                            reports.append(Report('W', toolname, part.code, message, line))
+                            reports.append(
+                                Report('W', toolname, part.code,
+                                       Report.substitution(what="wrong markup spacing",
+                                           with_what="{:+} chars".format(
+                                                     1 - len(markup_space_m.group(1)))),
+                                       node.child_nodes[2].code if len(node.child_nodes) > 1
+                                       else node.code))
                     continue
 
                 for child in part.child_nodes:
@@ -424,9 +438,10 @@ def indention(toolname, document, reports):
                             if part_len < ind_aim and offset != ind_aim:
                                 with_what += " or {:+} chars (col: {})".format(offset - part_len,
                                                                                offset + 1)
-                            message = Report.substitution(what="wrong indent", with_what=with_what)
-                            output = child.indent.code.copy().clear(False)
-                            reports.append(Report('E', toolname, output, message, child.code))
+                            reports.append(
+                                Report('E', toolname, child.indent.code.copy().clear(False),
+                                       Report.substitution(what="wrong indent",
+                                                           with_what=with_what), child.code))
 
     return reports
 
@@ -439,7 +454,7 @@ def kbd_pre(_):
     pattern_str = ''.join((
         # Keyboard
         # Modifier
-        r"^(Shift(?:\-|\Z))?(Ctrl(?:\-|\Z))?(Alt(?:\-|\Z))?((?:Cmd|OSKey)(?:\-|\Z))?",
+        r"^(Shift(?:\-|\Z))?(Ctrl(?:\-|\Z))?(Alt(?:\-|\Z))?((?:Cmd|OSKey)(?:\-|\Z))?(Fn(?:\-|\Z))?",
 
         # Note, shifted keys such as '!?:<>"' should not be included.
         r"((?:",
@@ -496,7 +511,6 @@ def kbd_pre(_):
 
 def kbd(toolname, document, reports, re_lib):
     """Report non-conforming uses of the :kbd: role."""
-
     valid_kbd = re_lib["valid_kbd"]
     repeat_kbd = re_lib["repeat_kbd"]
 
@@ -596,19 +610,19 @@ def leak_pre(_):
 
     # List
     re_lib["unordend"] = (
-        re.compile(r"\n *" + rst_parser.re_lib["bullet"].pattern
+        re.compile(r"\n *" + rst_parser.re_lib["bullet"][0].pattern
                    .replace(space_end, r"[A-Za-z]"), re.MULTILINE),
         Report.missing(what="space", where="after unordered list"))
 
     re_lib["ordend"] = (
-        re.compile(r"\n *" + rst_parser.re_lib["enum"].pattern
+        re.compile(r"\n *" + rst_parser.re_lib["enum"][0].pattern
                    .replace(space_end, r"[A-Za-z]").replace(r"\w", ""), re.MULTILINE),
         Report.missing(what="space", where="after ordered list"))
 
     # todo add line, field
     re_lib["overlist"] = (
-        re.compile(r"\n *(?:" + '|'.join((rst_parser.re_lib["bullet"].pattern,
-                                          rst_parser.re_lib["enum"].pattern)) + r")",
+        re.compile(r"\n *(?:" + '|'.join((rst_parser.re_lib["bullet"][0].pattern,
+                                          rst_parser.re_lib["enum"][0].pattern)) + r")",
                    re.MULTILINE),
         Report.missing(what="blank line", where="over list"))
 
@@ -686,8 +700,9 @@ def leak_pre(_):
     markup_keys.update({"arrow", "arrowlen", "dash", "escape"})
 
     # Merge Conflict
-    # FP = transition
-    re_lib["mc"] = (re.compile(r"(?:[<>|]{7} \.(?:r\d+?|mine))|(?:^={7}\n)", re.MULTILINE),
+    # FP transition with = char
+    re_lib["mc"] = (re.compile(r"^[<>|]{7} (\.(r\d+?|mine)|HEAD|[a-f0-9]{4,40}\b)" +
+                               r"|^={7}\n", re.MULTILINE),
         Report.existing(what="merge conflict"))
 
     node_pattern_map = (
@@ -714,7 +729,6 @@ def leak_pre(_):
 
 def leak(toolname, document, reports, re_lib, data):
     """Find pieces of leaked markup and suspicious patterns."""
-
     names = set(data[0].keys()) if not "*" in data[0].keys() else None
     for node in rst_walker.iter_node(document.body, names, leafs_only=True):
         mapper_portion = (data[0][node.node_name] if node.node_name in data[0].keys()
@@ -738,9 +752,10 @@ def leak(toolname, document, reports, re_lib, data):
                 for m in re.finditer(re_lib[key][0], part_str):
                     if on_mc_trans:
                         continue
-                    output = part.code.slice_match_obj(m, 0, True)
-                    line = Report.getline_offset(part.parent_node.code, output, 100)
-                    reports.append(Report('F', toolname, output, re_lib[key][1], line))
+                    reports.append(
+                        Report('F', toolname, part.code.slice_match(m, 0, True),
+                               re_lib[key][1])
+                        .set_line_offset(part.parent_node.code, 100))
 
 
     for node in rst_walker.iter_node(document.body, {"text", "sect", "field"}):
@@ -766,7 +781,6 @@ def leak(toolname, document, reports, re_lib, data):
             part_str = str(part.code)
             for key in data[1]:
                 for m in re.finditer(re_lib[key][0], part_str):
-                    output = part.code.slice_match_obj(m, 0, True)
                     if re_lib[key][2]:
                         where = rst_walker.write_out(re_lib[key][1][0], re_lib[key][1][1])
                         message = Report.existing(what="leaked", where=where)
@@ -804,15 +818,15 @@ def leak(toolname, document, reports, re_lib, data):
 
                     else:
                         message = re_lib[key][1]
-                    line = Report.getline_offset(part.parent_node.parent_node.code, output, 100)
-                    reports.append(Report('F', toolname, output, message, line))
+                    reports.append(
+                        Report('F', toolname, part.code.slice_match(m, 0, True), message)
+                        .set_line_offset(part.parent_node.parent_node.code, 100))
 
     return reports
 
 
 def markup_names(toolname, document, reports):
     """Find unknown/uncommon role and directive names."""
-
     roles = (
         # standard docutils
         'index', 'sub', 'sup', 'math',
@@ -825,14 +839,14 @@ def markup_names(toolname, document, reports):
     directives = (
         # standard docutils
         #   admonition
-        'admonition', 'hint', 'important', 'note', 'tip', 'warning',
+        'hint', 'important', 'note', 'tip', 'warning',
         #    other
         'container', 'figure', 'image', 'include', 'list-table', 'math',
         'parsed-literal', 'replace', 'rubric', 'unicode',
         # Sphinx custom
         'code-block', 'glossary', 'highlight', 'hlist', 'index', 'only', 'seealso', 'toctree',
         # Sphinx extension
-        'vimeo', 'youtube'
+        'reference', 'peertube', 'vimeo', 'youtube'
     )
 
     for node in rst_walker.iter_node(document.body, {"dir", "role", "block-quote"}):
@@ -840,29 +854,29 @@ def markup_names(toolname, document, reports):
             if node.name:
                 node_name_str = str(node.name.code).strip()
                 if node_name_str not in roles:
-                    message = ("uncommon role" if node_name_str in RSTParser.roles or
-                               node_name_str in RSTParser.roles_sphinx else "unknown role")
-                    reports.append(Report('E', toolname, node.name.code, message))
+                    reports.append(
+                        Report('E', toolname, node.name.code,
+                               "uncommon role" if node_name_str in RSTParser.roles or
+                               node_name_str in RSTParser.roles_sphinx else "unknown role"))
             else:
                 reports.append(Report('E', toolname, node.body.code, "default role"))
 
         elif node.node_name == "dir":
             name = str(node.name.code).strip() if node.name else node.name
             if name and name not in directives:
-                message = ("uncommon directive" if name in RSTParser.directives or
-                           name in RSTParser.directives_sphinx else "unknown directive")
-                reports.append(Report('E', toolname, node.name.code, message))
+                reports.append(
+                    Report('E', toolname, node.name.code,
+                           "uncommon directive" if name in RSTParser.directives or
+                           name in RSTParser.directives_sphinx else "unknown directive"))
 
             if name == "raw":
-                message = "raw directive"
-                reports.append(Report('F', toolname, node.name.code, message))
+                reports.append(Report('F', toolname, node.name.code, "raw directive"))
 
         else:
             # not at diff hunk start can be cut off def list.
             if node.code.start_lincol[0] != document.code.start_lincol[0]:
-                output = node.indent.code.copy().clear(False)
-                message = "block quote"
-                reports.append(Report('W', toolname, output, message))
+                reports.append(
+                    Report('W', toolname, node.indent.code.copy().clear(False), "block quote"))
 
     return reports
 
@@ -932,12 +946,14 @@ def structure_pre(_):
               {"output": False, "message": "transition not between text"}),
              ("dir(figure, list-table) - def-list \\ * + dir(figure, list-table) & def-list",
               {"output": True, "message": "image splitting definition list"}),
-             ("dir(admonition) = *[!class: refbox]",
-              {"severity": 'W', "output": True,
-               "message": "admonition directive without refbox class"}),
-             ("dir(admonition)[class: refbox] - !sect = !None",
-              {"output": True, "message": "refbox admonition not at section start"}),
-             ("sect + dir(figure, admonition, list-table, toctree) & !text, def-list, sect = !None",
+             ("dir(reference) - !sect = !None",
+              {"output": True, "message": "reference box not at section start"}),
+             ("dir(reference) / head > !None",
+              {"output": False, "message": "reference box head misplaced content"}),
+             ("dir(reference) / body > * == !field-list + !None",
+              {"output": False,
+               "message": "reference box body should contain (only) a field-list"}),
+             ("sect + dir(figure, reference, list-table, toctree) & !text, def-list, sect = !None",
               {"output": False, "message": "section not starting with a text"}),
              ("bullet-list, enum-list - !text = !None",
               {"output": True, "message": "list without an introductory text"}),
@@ -948,7 +964,9 @@ def structure_pre(_):
                " consider moving them into a list-table"}),
              ("- sect & {1} = None \\ * + sect && {2} = !None",
               {"output": True, "message": "three or more headings in a row"}),
-             ("dir(figure) / body = None",
+             ("dir(rubric) << !document",
+              {"output": True, "message": "nested rubric"}),
+             ("dir(figure) / body = None \\ * + !None",
               {"output": True, "message": "figure without a caption"}),
              ("strong, emphasis << sect",
               {"output": True, "message": "font styling in heading"}),
@@ -961,6 +979,8 @@ def structure_pre(_):
               {"output": True, "message": "seealso admonition not at section end"}),
              ("dir(index) - target, comment",
               {"output": True, "message": "index directive not at top"}),
+             ("dir(container) / head > None",
+              {"output": False, "message": "container directive without head"}),
             )
     operators_start = {
         "^": " + None \\ *",
@@ -1208,15 +1228,16 @@ def structure(toolname, document, reports, data):
                         if (not operator_next or
                                 waypoint_active["operator"] == "==" or
                                 (waypoint_con and waypoint_con["operator"] == "==")):
-                            output = node
+                            node_output = node
                             if report_info["output"]:
                                 if node_active:
-                                    output = node_active
+                                    node_output = node_active
                                 elif node_con:
-                                    output = node_con
-                            output = output.code.copy().clear(True)
-                            reports.append(Report(report_info.get("severity", 'I'), toolname,
-                                                  output, report_info["message"]))
+                                    node_output = node_con
+                            reports.append(
+                                Report(report_info.get("severity", 'I'), toolname,
+                                       node_output.code.copy().clear(True),
+                                       report_info["message"]))
 
                             break
 
