@@ -49,12 +49,12 @@ def abbreviation_pre(_):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*",
-            "math": "*", "youtube": "*", "vimeo": "*"
+            "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*", "comment": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "math": "*", "term": "*"
+            "kbd": "*", "math": "*", "term": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -120,10 +120,11 @@ def abbreviation(toolname, document, reports, data, config):
             for entry, loc in data["explanations"][document.code.filename]:
                 if re.match(word_re, entry):
                     if word.start_pos < loc:
-                        message = Report.existing(what="abbreviation",
-                                                  where="before its explanation")
-                        line = Report.getline_punc(part.code, word, 50, 30)
-                        reports.append(Report('I', toolname, word, message, line))
+                        reports.append(
+                            Report('I', toolname, word,
+                                   Report.existing(what="abbreviation",
+                                                   where="before its explanation"))
+                            .set_line_punc(part.code, 50, 30))
                     break
 
             else:
@@ -160,9 +161,10 @@ def abbreviation(toolname, document, reports, data, config):
                     where = None
                     severity = 'W'
 
-                message = Report.missing(what="explanation", where=where)
-                line = Report.getline_punc(part.code, word, 50, 30)
-                reports.append(Report(severity, toolname, word, message, line))
+                reports.append(
+                    Report(severity, toolname, word,
+                           Report.missing(what="explanation", where=where))
+                    .set_line_punc(part.code, 50, 30))
 
     return reports
 
@@ -198,34 +200,40 @@ def article(toolname, document, reports, re_lib, data):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*", "toctree": "*",
-            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*"
+            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*",
-        "role": {"math": "*"},
+        "comment": "*", "doctest": "*", "target": "*",
+        "role": {"math": "*", "default": "*"},
         "literal": "*", "standalone": "*"
     }
 
     is_a = None
     for part in rst_walker.iter_nodeparts_instr(document.body, instr_pos, instr_neg, False):
-        if part.child_nodes.is_empty():
-            for word in segmenter.iter_word(part.code, filter_numbers=False):
+        if not part.child_nodes.is_empty():
+            if part.parent_node.node_name in {"def", "bullet", "enum", "field", "line"}:
+                is_a = None
+            continue
+
+        for sen, stop in segmenter.iter_sentence(part.code):
+            for word in segmenter.iter_word(sen, filter_numbers=False):
                 word_str = str(word).strip()
                 if len(word) < 3 and word_str in {"a", "A", "an", "An"}:
                     is_a = bool(word_str in {"a", "A"})
                 elif is_a is not None:
                     if re.match(digit_re, word_str):
                         if not is_a:
-                            message = Report.existing(what="an", where="before digit")
-                            line = Report.getline_punc(document.body.code, word, 50, 30)
-                            reports.append(Report('E', toolname, word, message, line))
+                            reports.append(
+                                Report('E', toolname, word,
+                                       Report.existing(what="an", where="before digit"))
+                                .set_line_punc(document.body.code, 50, 30))
                     else:
                         is_cons =  bool(not re.match(vowel_re, word_str))
                         is_cons_sound = is_cons
                         key = "a" if is_cons else "an"
                         token = word
                         if token_m := re.match(token_re, word_str):
-                            token = word.slice_match_obj(token_m, 1, True)
+                            token = word.slice_match(token_m, 1, True)
                         if (len(token) == 1 or part_of_speech.isacr(token) or
                                 part_of_speech.isabbr(token)):
                             if word_str[0].lower() in data[key]["letter"]:
@@ -245,21 +253,21 @@ def article(toolname, document, reports, re_lib, data):
                                     break
 
                         if is_a != is_cons_sound:
-                            where = " ".join(("before", "consonant" if is_cons_sound else "vowel",
-                                              "sound" if is_cons_sound != is_cons else ""))
-                            message = Report.existing(what="a" if is_a else "an",
-                                                      where=where)
-                            line = Report.getline_punc(document.body.code, word, 50, 30)
-                            reports.append(Report('E', toolname, word, message, line))
+                            reports.append(
+                                Report('E', toolname, word,
+                                       Report.existing(what="a" if is_a else "an",
+                                           where=" ".join(("before",
+                                                    "consonant" if is_cons_sound else "vowel",
+                                                    "sound" if is_cons_sound != is_cons else ""))))
+                                .set_line_punc(document.body.code, 50, 30))
 
                     is_a = None
 
-            if part.parent_node.node_name == "role":
+            if stop:
                 is_a = None
 
-        else:
-            if part.parent_node.node_name in {"def", "bullet", "enum", "field", "line"}:
-                is_a = None
+        if part.parent_node.node_name == "role":
+            is_a = None
 
     return reports
 
@@ -320,14 +328,14 @@ def collocation_pre(_):
             terms.append((list(" ".join(group) for group in result), word))
 
     args = dict()
-    args["config"] = listsearch.parse_flags("BIO")
-    args["data"] = listsearch.compile_terms(terms, {"flags": args["config"]})
+    args["config"] = {}
+    args["data"] = listsearch.compile_terms(terms, {"flags": listsearch.parse_flags("BIO")})
 
     return args
 
 
 def collocation(toolname, document, reports, data, config):
-    return listsearch.search(toolname, document, reports, data, config)
+    return listsearch.search_char(toolname, document, reports, data, config)
 
 
 def grammar_pre(_):
@@ -392,19 +400,18 @@ def hyphen_pre(_):
             terms.append([re.sub(dash_re, " ", word), word])
 
     args = dict()
-    args["config"] = listsearch.parse_flags("BIO")
-    args["data"] = listsearch.compile_terms(terms, {"flags": args["config"]})
+    args["config"] = {}
+    args["data"] = listsearch.compile_terms(terms, {"flags": listsearch.parse_flags("BIO")})
 
     return args
 
 
 def hyphen(toolname, document, reports, data, config):
-    return listsearch.search(toolname, document, reports, data, config)
+    return listsearch.search_char(toolname, document, reports, data, config)
 
 
 def metric(toolname, document, reports):
     """Measure length of segments like paragraphs, sentences and words."""
-
     # source: https://www.gov.uk/guidance/content-design/writing-for-gov-uk
     conf = {
         "sect_len": 69,
@@ -418,26 +425,29 @@ def metric(toolname, document, reports):
     def compare(node_cur, sen_full, counter, reports, sub_para=False, is_last=False):
         if node_cur.node_name == "sect":
             if counter["sect"] > conf["sect_len"]:
-                output = node_cur.code.copy().clear(True)
-                message = Report.quantity(what="long heading",
-                                          how="{0}/{1} letters".format(
-                                              counter["sect"], conf["sect_len"]))
-                reports.append(Report('I', toolname, output, message, node_cur.code))
+                reports.append(
+                    Report('I', toolname, node_cur.code.copy().clear(True),
+                           Report.quantity(what="long heading",
+                               how="{0}/{1} letters".format(
+                                   counter["sect"], conf["sect_len"])),
+                           node_cur.code))
 
         else:
             if counter["sen"] > conf["sen_len"]:
-                output = sen_full.copy().clear(True)
-                message = Report.quantity(what="long sentence",
-                                          how="{0}/{1} words".format(
-                                              counter["sen"], conf["sen_len"]))
-                reports.append(Report('I', toolname, output, message, sen_full))
+                reports.append(
+                    Report('I', toolname, sen_full.copy().clear(True),
+                           Report.quantity(what="long sentence",
+                               how="{0}/{1} words".format(
+                                   counter["sen"], conf["sen_len"])),
+                           sen_full))
             if not sub_para:
                 if counter["para"] > conf["para_long"]:
-                    output = node_cur.code.copy().clear(True)
-                    message = Report.quantity(what="long paragraph",
-                                              how="{0}/{1} sentences".format(
-                                                  counter["para"], conf["para_long"]))
-                    reports.append(Report('I', toolname, output, message, node_cur.code))
+                    reports.append(
+                        Report('I', toolname, node_cur.code.copy().clear(True),
+                               Report.quantity(what="long paragraph",
+                                   how="{0}/{1} sentences".format(
+                                       counter["para"], conf["para_long"])),
+                               node_cur.code))
                 check = False
                 if counter["para"] <= conf["para_short"] and counter["para"] != 0:
                     counter["para_short"] += 1
@@ -445,14 +455,14 @@ def metric(toolname, document, reports):
                     check = True
                 if (check or is_last):
                     if counter["para_short"] > 1:
-                        output = node_cur.code.copy().clear(True)
-                        message = Report.quantity(what="multiple short paragraph",
-                                                  how="{0}/{1} paragraphs".format(
-                                                      counter["para_short"], 1))
-
-                        reports.append(Report('I', toolname, output, message, node_cur.code
-                                              if not (node_cur.code.isspace() and node_cur.prev)
-                                              else node_cur.prev.code))
+                        reports.append(
+                            Report('I', toolname, node_cur.code.copy().clear(True),
+                                   Report.quantity(what="multiple short paragraph",
+                                       how="{0}/{1} paragraphs".format(
+                                           counter["para_short"], 1)),
+                                   node_cur.code
+                                   if not (node_cur.code.isspace() and node_cur.prev)
+                                   else node_cur.prev.code))
                         counter["para_short"] = 0
                     elif counter["para"] != 0:
                         counter["para_short"] = 0
@@ -467,26 +477,26 @@ def metric(toolname, document, reports):
     instr_neg = {
         "dir": {
             "*": ["head"], "include": "*", "index": "*", "toctree": "*",
-            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*"
+            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "def": {"*": ["head"]},
         "substdef": {"*": ["head"], "unicode": "*", "replace": "*"},
-        "target": "*", "comment": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "menuselection": "*", "math": "*"
+            "kbd": "*", "menuselection": "*", "math": "*", "default": "*"
         },
         "standalone": "*", "literal": "*", "substitution": "*"
     }
     node_cur = None
     node_prev = None
-    is_open = False
+    stop = None
     sen_full = None
     counter = dict.fromkeys({"sect", "sen", "para", "para_short"}, 0)
 
     for part in rst_walker.iter_nodeparts_instr(document.body, instr_pos, instr_neg, False):
         if node_cur is None or part.code.end_pos > node_cur.code.end_pos:
             if node_cur:
-                if is_open and sen_full and not sen_full.isspace():
+                if not stop and sen_full and not sen_full.isspace():
                     counter["para"] += 1
                 node_prev = node_cur
                 is_last = bool(rst_walker.is_of(node_cur.parent_node,
@@ -497,6 +507,7 @@ def metric(toolname, document, reports):
                 if is_last:
                     counter["para_short"] = 0
                     node_prev = None
+                    sen_full = None
 
                 if (rst_walker.is_of(part, "dir", {"code-block", "default"}) or
                         rst_walker.is_of(part, "comment")):
@@ -521,20 +532,22 @@ def metric(toolname, document, reports):
                 reports = compare(node_prev, sen_full, counter, reports, is_last=True)
                 counter["para_short"] = 0
                 node_prev = None
+                sen_full = None
 
         if node_cur and part.child_nodes.is_empty():
             if part.code.end_pos <= node_cur.code.end_pos:
                 if node_cur.node_name == "sect":
                     counter["sect"] += len(part.code)
                 else:
-                    for sen, is_open in segmenter.iter_sentence(part.code, output_openess=True):
+                    for sen, stop in segmenter.iter_sentence(part.code):
                         for word in segmenter.iter_word(sen):
                             counter["sen"] += 1
                             if len(word) >= conf["word_len"]:
-                                message = Report.quantity(what="long word",
-                                                          how="{0}/{1} letters".format(
-                                                              len(word), conf["word_len"]))
-                                reports.append(Report('I', toolname, word, message))
+                                reports.append(
+                                    Report('I', toolname, word,
+                                           Report.quantity(what="long word",
+                                               how="{0}/{1} letters".format(
+                                                   len(word), conf["word_len"]))))
 
                         if not sen_full:
                             sen_full = sen
@@ -542,7 +555,7 @@ def metric(toolname, document, reports):
                             sen_full = document.body.code.slice(
                                            sen_full.start_lincol, sen.end_lincol, True)
 
-                        if not is_open:
+                        if stop:
                             reports = compare(node_cur, sen_full, counter, reports, True)
                             counter["sen"] = 0
                             counter["para"] += 1
@@ -556,7 +569,7 @@ def metric(toolname, document, reports):
                         sen_full = None
 
     if node_cur:
-        if is_open and sen_full and not sen_full.isspace():
+        if not stop and sen_full and not sen_full.isspace():
             counter["para"] += 1
         reports = compare(node_cur, sen_full, counter, reports, is_last=True)
 
@@ -565,8 +578,7 @@ def metric(toolname, document, reports):
 
 def overuse_pre(toolname):
     config = dict()
-    config.update(monostyle_io.get_override(__file__, toolname, "threshold_min", 2.0))
-    config.update(monostyle_io.get_override(__file__, toolname, "threshold_severe", 3.1))
+    config.update(monostyle_io.get_override(__file__, toolname, "thresholds", (2.0, 3.1)))
     return {"config": config}
 
 
@@ -574,8 +586,7 @@ def overuse(toolname, document, reports, config):
     """Overuse of words. Filter with markup, subjects after an determiner,
     transitions at sentence start, the file path and stopwords.
     """
-    threshold_min = config["threshold_min"]
-    threshold_severe = config["threshold_severe"]
+    thresholds = config["thresholds"]
     distance_min = 5
     distance_max = 80
     modifier_as_topic = True
@@ -585,10 +596,12 @@ def overuse(toolname, document, reports, config):
         words.remove(word_str)
 
     def evaluate(document, reports, words):
-        def add_report(document, reports, word, is_severe, count):
-            message = Report.quantity(what="overused word", how=str(count) + " times")
-            line = Report.getline_punc(document.code, word, 50, 30)
-            reports.append(Report('I' if not is_severe else 'W', toolname, word, message, line))
+        def add_report(document, reports, word, thresholds, score, count):
+            reports.append(
+                Report(Report.map_severity(thresholds, score), toolname, word,
+                       Report.quantity(what="overused word",
+                                       how=str(count) + " times"))
+                .set_line_punc(document.code, 50, 30))
 
             return reports
 
@@ -604,10 +617,9 @@ def overuse(toolname, document, reports, config):
                     buf = instance
                 distance = instance[2] - buf[2]
                 if distance > distance_max:
-                    if score >= threshold_min:
+                    if score >= thresholds[0]:
                         reports = add_report(document, reports, instance[0],
-                                             bool(score >= threshold_severe),
-                                             index - index_last + 1)
+                                             thresholds, score, index - index_last + 1)
                     buf = None
                     score = 0
                     index_last = index
@@ -617,9 +629,9 @@ def overuse(toolname, document, reports, config):
                                                       (distance - distance_min), 2)) + 1)
                     buf = instance
 
-            if score >= threshold_min:
-                reports = add_report(document, reports, instance[0],
-                                     bool(score >= threshold_severe), index - index_last + 1)
+            if score >= thresholds[0]:
+                reports = add_report(document, reports, instance[0], thresholds, score,
+                                     index - index_last + 1)
         words.reset()
         return reports
 
@@ -634,12 +646,12 @@ def overuse(toolname, document, reports, config):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*",
-            "math": "*", "youtube": "*", "vimeo": "*"
+            "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "math": "*"
+            "kbd": "*", "math": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -675,7 +687,7 @@ def overuse(toolname, document, reports, config):
         if word:
             # add per average word length in skipped code
             counter += (part.code.start_pos - word.end_pos) // 6
-        for sen, is_open in segmenter.iter_sentence(part.code, output_openess=True):
+        for sen, stop in segmenter.iter_sentence(part.code):
             for word in segmenter.iter_word(sen):
                 counter += 1
                 word_str = str(word).lower()
@@ -731,7 +743,7 @@ def overuse(toolname, document, reports, config):
 
                 is_first = False
 
-            if not is_open:
+            if stop:
                 is_first = True
 
         if not part.next and not part.parent_node.next:
@@ -771,12 +783,12 @@ def search_pure(toolname, document, reports, re_lib, config):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*",
-            "math": "*", "youtube": "*", "vimeo": "*"
+            "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "math": "*"
+            "kbd": "*", "math": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -785,10 +797,10 @@ def search_pure(toolname, document, reports, re_lib, config):
         for pattern, message in re_lib.values():
             part_str = str(part.code)
             for m in re.finditer(pattern, part_str):
-                output = part.code.slice_match_obj(m, 0, True)
-                line = Report.getline_punc(document.body.code, output, 50, 30)
-                reports.append(Report(config.get("severity"), toolname,
-                                      output, message, line))
+                reports.append(
+                    Report(config.get("severity"), toolname,
+                           part.code.slice_match(m, 0, True), message)
+                    .set_line_punc(document.body.code, 50, 30))
 
     return reports
 
@@ -801,7 +813,6 @@ def repeated_pre(toolname):
 
 def repeated(toolname, document, reports, config):
     """Find repeated words e.g. the the example."""
-
     def stemmer_patch(porter_stemmer, word_lower):
         """Distinguish some words."""
         # on vs. one
@@ -826,12 +837,12 @@ def repeated(toolname, document, reports, config):
         "dir": {
             "figure": ["head"],
             "code-block": "*", "default": "*", "include": "*", "index": "*", "toctree": "*",
-            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*"
+            "parsed-literal": "*", "math": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*",
+        "comment": "*", "doctest": "*", "target": "*",
         "role": {
-            "kbd": "*", "menuselection": "*", "math": "*"
+            "kbd": "*", "menuselection": "*", "math": "*", "default": "*"
         },
         "literal": "*", "standalone": "*"
     }
@@ -851,7 +862,7 @@ def repeated(toolname, document, reports, config):
 
                 buf.append("")
 
-            for sen, is_open in segmenter.iter_sentence(part.code, output_openess=True):
+            for sen, stop in segmenter.iter_sentence(part.code):
                 for word in segmenter.iter_word(sen):
                     word_lower = str(word).lower()
                     word_stem = stemmer_patch(porter_stemmer, word_lower)
@@ -865,11 +876,11 @@ def repeated(toolname, document, reports, config):
                             if distance >= ignore_pre_pro[1] and word_stem in ignore_pre_pro[0]:
                                 continue
 
-                            severity = 'W' if distance == 0 else 'I'
-                            message = Report.quantity(what="repeated words",
-                                                      how=str(distance) + " words in between")
-                            line = Report.getline_punc(document.body.code, word, 50, 30)
-                            reports.append(Report(severity, toolname, word, message, line))
+                            reports.append(
+                                Report('W' if distance == 0 else 'I', toolname, word,
+                                       Report.quantity(what="repeated words",
+                                                       how=str(distance) + " words in between"))
+                                .set_line_punc(document.body.code, 50, 30))
                             break
 
                     if len(buf) == buf_size:
@@ -877,7 +888,7 @@ def repeated(toolname, document, reports, config):
 
                     buf.append(word_stem)
 
-                if not is_open:
+                if stop:
                     buf.clear()
 
             if rst_walker.is_of(part, "text") and part.parent_node.indent:
