@@ -10,7 +10,6 @@ import re
 
 import monostyle.util.monostyle_io as monostyle_io
 from monostyle.util.report import Report
-from monostyle.util.fragment import Fragment
 import monostyle.rst_parser.walker as rst_walker
 from monostyle.util.part_of_speech import PartofSpeech
 from monostyle.util.char_catalog import CharCatalog
@@ -84,7 +83,7 @@ def mark_pre(_):
         Report.existing(what="opening punctuation", where="at line end"))
 
     # Style
-    re_lib["optplur"] = (re.compile(r"\w/s\b"),
+    re_lib["optplur"] = (re.compile(r"\w/e?s\b"),
         Report.substitution(what="optional plural", with_what="(s)"))
 
     # FP: cut toctree
@@ -149,7 +148,7 @@ def mark(toolname, document, reports, re_lib):
     instr_neg = {
         "dir": {
             "figure": ["head"], "toctree": "*", "include": "*", "index": "*",
-            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*"
+            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
         "doctest": "*",
@@ -166,9 +165,9 @@ def mark(toolname, document, reports, re_lib):
                 for m in re.finditer(pattern, part_str):
                     if m.start() == 0 and key == "closesol" and part.parent_node.prev:
                         continue
-                    output = part.code.slice_match_obj(m, 0, True)
-                    line = Report.getline_punc(document.body.code, output, 50, 30)
-                    reports.append(Report('W', toolname, output, value[1], line))
+                    reports.append(
+                        Report('W', toolname, part.code.slice_match(m, 0, True), value[1])
+                        .set_line_punc(document.body.code, 50, 30))
 
 
     instr_pos = {
@@ -178,9 +177,10 @@ def mark(toolname, document, reports, re_lib):
     instr_neg = {
         "dir": {
             "figure": ["head"], "toctree": "*", "include": "*", "index": "*",
-            "admonition": ["head"], "hint": ["head"], "important": ["head"],
-            "note": ["head"], "tip": ["head"], "warning": ["head"], "rubric": ["head"],
-            "code-block": "*", "default": "*", "highlight": "*", "youtube": "*", "vimeo": "*"
+            "hint": ["head"], "important": ["head"], "note": ["head"],
+            "tip": ["head"], "warning": ["head"], "rubric": ["head"],
+            "code-block": "*", "default": "*", "highlight": "*",
+            "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": "*",
         "def": {"*": ["head"]}, "target": {"*": ["head"]},
@@ -214,26 +214,26 @@ def mark(toolname, document, reports, re_lib):
                         continue
                     if not is_sentence(part, par_node, instr_pos, instr_neg, space_re):
                         continue
-                    output = part.code.copy().clear(False)
-                    message = re_lib["puncend"][1][0].format("paragraph")
-                    line = Report.getline_offset(document.body.code, output, 100, False)
-                    reports.append(Report('W', toolname, output, message, line))
+                    reports.append(
+                        Report('W', toolname, part.code.copy().clear(False),
+                               re_lib["puncend"][1][0].format("paragraph"))
+                        .set_line_offset(document.body.code, 100, False))
 
                 else:
                     if comma_m := re.search(comma_re, part_str):
-                        output = part.code.slice_match_obj(comma_m, 0, True)
-                        message = (re_lib["commaend"][1]
-                                   .format(rst_walker.write_out(part.parent_node.node_name) +
-                                           " " + part.node_name))
-                        reports.append(Report('W', toolname, output, message))
+                        reports.append(
+                            Report('W', toolname, part.code.slice_match(comma_m, 0, True),
+                                   re_lib["commaend"][1].format(
+                                       rst_walker.write_out(part.parent_node.node_name) +
+                                                            " " + part.node_name)))
 
         elif rst_walker.is_of(part, {"role", "hyperlink"}, "*", "head"):
             if re.search(punc_end_re, str(part.code)):
-                output = part.code.copy().clear(False)
-                message = (re_lib["puncend"][1][1]
-                           .format(rst_walker.write_out(part.parent_node.node_name) +
-                                   " " + part.node_name))
-                reports.append(Report('W', toolname, output, message))
+                reports.append(
+                    Report('W', toolname, part.code.copy().clear(False),
+                           re_lib["puncend"][1][1].format(
+                               rst_walker.write_out(part.parent_node.node_name) +
+                                                    " " + part.node_name)))
 
     return reports
 
@@ -289,7 +289,7 @@ def number_pre(_):
     re_lib["percentlimit"] = (re.compile(r"\D(?:0|100)%"),
         Report.option(what="percentage limits", with_what="written out no or fully"))
 
-    #-----------------
+    # ------------------------------------------------------------------------
 
     re_lib["dimension"] = (re.compile(r"\b[0-9]d\b"),
         Report.misformatted(what="lowercase dimension letter"))
@@ -318,7 +318,6 @@ def number_pre(_):
 
 def number(toolname, document, reports, re_lib):
     """Check for numbers and units formatting."""
-
     instr_pos = {
         "field": {"*": ["body"]},
         "*": {"*": ["head", "body"]}
@@ -326,11 +325,11 @@ def number(toolname, document, reports, re_lib):
     instr_neg = {
         "dir": {
             "figure": ["head"],
-            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*"
+            "code-block": "*", "default": "*", "youtube": "*", "vimeo": "*", "peertube": "*"
         },
         "substdef": {"image": ["head"], "unicode": "*", "replace": "*"},
-        "doctest": "*", "comment": "*",
-        "role": {"kbd": "*"},
+        "comment": "*", "doctest": "*", "target": "*",
+        "role": {"kbd": "*", "default": "*"},
         "literal": "*", "standalone": "*", "footref": "*", "citref": "*"
     }
 
@@ -344,23 +343,18 @@ def number(toolname, document, reports, re_lib):
                          rst_walker.is_of(part, "dir", "math"))):
                     continue
 
-                output = part.code.slice_match_obj(m, 0, True)
-                line = Report.getline_punc(document.body.code, output, 50, 30)
-                reports.append(Report('W', toolname, output, value[1], line))
+                reports.append(
+                    Report('W', toolname, part.code.slice_match(m, 0, True), value[1])
+                    .set_line_punc(document.body.code, 50, 30))
 
     return reports
 
 
 def pairs_pre(toolname):
     args = dict()
-    re_lib = dict()
-
     # FP/FN: s' closing
     # FP: cut heading line
-
-    re_lib["pairchar"] = re.compile(r"[\(\[\{\)\]\}\]\"]|(?<!\w)'|(?<![sS])'(?!\w)")
-
-    args["re_lib"] = re_lib
+    args["re_lib"] = {"pairchar": re.compile(r"[\(\[\{\)\]\}\]\"]|(?<!\w)'|(?<![sS])'(?!\w)")}
 
     # Max number of lines between the open and close mark.
     args["config"] = dict(monostyle_io.get_override(__file__, toolname, "max_line_span", 2))
@@ -370,17 +364,17 @@ def pairs_pre(toolname):
 
 def pairs(toolname, document, reports, re_lib, config):
     """Check if pairs of inline markup, brackets, quote marks are closed."""
-
     instr_pos = {
         "sect": {"*": ["name"]},
         "field": {"*": ["name", "body"]},
         "*": {"*": ["head", "body"]}
     }
     instr_neg = {
-        "dir": {"code-block": "*", "default": "*", "math": "*", "youtube": "*", "vimeo": "*"},
+        "dir": {"code-block": "*", "default": "*", "math": "*",
+                "youtube": "*", "vimeo": "*", "peertube": "*"},
         "substdef": {"unicode": "*", "replace": "*"},
-        "doctest": "*", "target": "*", "comment": "*",
-        "role": {"kbd": "*", "menuselection": "*", "math": "*"},
+        "comment": "*", "doctest": "*", "target": "*",
+        "role": {"kbd": "*", "menuselection": "*", "math": "*", "default": "*"},
         "literal": "*", "standalone": "*"
     }
 
@@ -390,36 +384,31 @@ def pairs(toolname, document, reports, re_lib, config):
     pairs_map = {')': '(', ']': '[', '}': '{'}
 
     for part in rst_walker.iter_nodeparts_instr(document.body, instr_pos, instr_neg):
-        for line in part.code.splitlines():
-            line_str = str(line)
-            for pair_m in re.finditer(pair_re, line_str):
-                pair_char = pair_m.group(0)
-                for index, entry in enumerate(reversed(stack)):
-                    if (entry[0] == pair_char or
-                            (pair_char in pairs_map and entry[0] == pairs_map[pair_char])):
+        part_str = str(part.code)
+        for pair_m in re.finditer(pair_re, part_str):
+            pair_char = part.code.slice_match(pair_m, 0, True)
+            pair_char_str = pair_m.group(0)
+            for index, entry in enumerate(reversed(stack)):
+                if (entry[0] == pair_char_str or
+                        (pair_char_str in pairs_map and entry[0] == pairs_map[pair_char_str])):
 
-                        if (max_line_span is not None and
-                                line.start_lincol[0] - entry[1][0] > max_line_span):
-                            lincol_abs = line.loc_to_abs((0, pair_m.start(0)))
-                            message = "long span"
-                            message += " - " + str(lincol_abs[0] + 1) + ","
-                            message += str(lincol_abs[1] + 1)
-                            output = Fragment(document.code.filename, entry[0], -1,
-                                              start_lincol=entry[1])
-                            reports.append(Report('W', toolname, output, message))
+                    if (max_line_span is not None and
+                            pair_char.start_lincol[0] - entry[1].start_lincol[0] > max_line_span):
+                        reports.append(
+                            Report('W', toolname, entry[1], "long span - {0},{1}"
+                                   .format(pair_char.start_lincol[0] + 1,
+                                           pair_char.start_lincol[1] + 1)))
 
-                        # invert index
-                        stack.pop(len(stack) - 1 - index)
-                        break
+                    # invert index
+                    stack.pop(len(stack) - 1 - index)
+                    break
 
-                else:
-                    stack.append((pair_char, line.loc_to_abs((0, pair_m.start(0)))))
+            else:
+                stack.append((pair_char_str, pair_char))
 
     if len(stack) != 0:
-        message = "unclosed pairs"
         for entry in stack:
-            output = Fragment(document.code.filename, entry[0], -1, start_lincol=entry[1])
-            reports.append(Report('W', toolname, output, message))
+            reports.append(Report('W', toolname, entry[1], "unclosed pairs"))
 
     if max_line_span is not None:
         for node in rst_walker.iter_node(document.body,
@@ -432,14 +421,14 @@ def pairs(toolname, document, reports, re_lib, config):
 
             if (node.body_start.code.end_lincol[0] - node.body_end.code.start_lincol[0] >
                     max_line_span):
-                message = "long span"
-                message += " - " + str(node.body_start.code.end_lincol[0] + 1)
-                message += "," + str(node.body_end.code.start_lincol[1] + 1)
-                reports.append(Report('W', toolname, node.body_start.code, message))
+                reports.append(
+                    Report('W', toolname, node.body_start.code,
+                           "long span - {0},{1}".format(
+                                node.body_end.code.start_lincol[0] + 1,
+                                node.body_end.code.start_lincol[1] + 1)))
 
             if node.body_start.code.end_pos == node.body_end.code.start_pos:
-                message = "zero span"
-                reports.append(Report('W', toolname, node.code, message))
+                reports.append(Report('W', toolname, node.code, "zero span"))
 
     return reports
 
@@ -467,18 +456,18 @@ def whitespace_pre(_):
 
 def whitespace(toolname, document, reports, re_lib):
     """Check whitespace chars."""
-
     text = str(document.code)
     for key, value in re_lib.items():
         if key in {"multispace", "multispacestart"}:
             continue
         pattern = value[0]
         for m in re.finditer(pattern, text):
-            output = document.body.code.slice_match_obj(m, 0, True)
-            line = Report.getline_punc(document.body.code, output, 50, 30)
-            fix = document.body.code.slice_match_obj(m, 1, True)
+            fix = document.body.code.slice_match(m, 1, True)
             fix.replace_fill(value[2])
-            reports.append(Report('W', toolname, output, value[1], line, fix))
+            reports.append(
+                Report('W', toolname, document.body.code.slice_match(m, 0, True),
+                       value[1], fix=fix)
+                .set_line_punc(document.body.code, 50, 30))
 
     multi_start_re = re_lib["multispacestart"][0]
     multi_re = re_lib["multispace"][0]
@@ -488,18 +477,18 @@ def whitespace(toolname, document, reports, re_lib):
         node_str = str(node.code)
         if node.prev:
             if multi_start_m := re.match(multi_start_re, node_str):
-                output = node.code.slice_match_obj(multi_start_m, 1, True)
-                line = Report.getline_punc(document.body.code, output, 50, 30)
-                fix = output.copy().replace_fill(value[2])
-                reports.append(Report('W', toolname, output, re_lib["multispacestart"][1],
-                                      line, fix if not is_cell else None))
+                output = node.code.slice_match(multi_start_m, 1, True)
+                reports.append(
+                    Report('W', toolname, output, re_lib["multispacestart"][1],
+                           fix=output.copy().replace_fill(value[2]) if not is_cell else None)
+                    .set_line_punc(document.body.code, 50, 30))
 
         for multi_m in re.finditer(multi_re, node_str):
-            output = node.code.slice_match_obj(multi_m, 1, True)
-            line = Report.getline_punc(document.body.code, output, 50, 30)
-            fix = output.copy().replace_fill(value[2])
-            reports.append(Report('W', toolname, output, re_lib["multispace"][1],
-                                  line, fix if not is_cell else None))
+            output = node.code.slice_match(multi_m, 1, True)
+            reports.append(
+                Report('W', toolname, output, re_lib["multispace"][1],
+                       fix=output.copy().replace_fill(value[2]) if not is_cell else None)
+                .set_line_punc(document.body.code, 50, 30))
 
     return reports
 
