@@ -193,7 +193,7 @@ def process_para(node, ind_offset, options):
                 if m.start() == 0:
                     continue
 
-            space = part.code.slice_match_obj(m, 0, True)
+            space = part.code.slice_match(m, 0, True)
             word = node.code.slice(last, space.start_pos, True)
             demerits = 0
             if part.node_name == "id_start":
@@ -359,7 +359,6 @@ def pos_weight(part_of_speech, word):
     a line before/after this part of speech tag,
     The structure has to by the same as the pos data.
     """
-
     weights = {
         "noun": [0, 0],
         "verb": [0, 0],
@@ -380,8 +379,8 @@ def pos_weight(part_of_speech, word):
         "interjection": [0, 0],
         "determiner": {
             "article": [2, -5],
-            "demonstrative": [0, 0],
-            "possessive": [0, 0],
+            "demonstrative": [1, -3],
+            "possessive": [1, -3],
             "quantifier": [0, -3],
             "numeral": {
                 "cardinal": [-1, -3],
@@ -523,7 +522,7 @@ def measure_indent(node):
                 # indent style
                 if (rst_walker.is_of(par_node,
                         {"dir", "target", "substdef", "footdef", "citdef"}, "*", "head")):
-                    ind_cur = par_node.parent_node.name_start.code.start_lincol[1] + 3
+                    ind_cur = par_node.parent_node.indent.code.end_lincol[1] + 3
                 else:
                     ind_cur = node.code.start_lincol[1]
         else:
