@@ -65,14 +65,14 @@ class Fragment():
     def get_end(self, pos_lincol):
         return self.end_pos if pos_lincol else self.end_lincol
 
-    #--------------------
-    # Filename
+
+    # -- Filename ------------------------------------------------------------
 
     def has_consistent_filenames(self):
         return True
 
-    #--------------------
-    # Content
+
+    # -- Content -------------------------------------------------------------
 
     def extend(self, new_content, keep_end=False):
         """Adds lines to the content. Join first with last if not newline end."""
@@ -84,22 +84,21 @@ class Fragment():
         if isinstance(new_content, str):
             new_content = [line for line in new_content.splitlines(keepends=True)]
 
-        is_nl_end = bool(not self.content or self.content[-1].endswith('\n'))
-        if is_nl_end:
-            self.content.append(new_content[0])
+        is_empty = bool(not self.content)
+        is_nl_end = bool(not is_empty and self.content[-1].endswith('\n'))
+        if is_nl_end or is_empty:
+            self.content.extend(new_content)
         else:
             self.content[-1] += new_content[0]
+            self.content.extend(new_content[1:])
 
         if not keep_end:
             self.end_pos += sum(map(len, new_content))
 
             if self.end_lincol:
-                self.end_lincol = (self.end_lincol[0] + (len(new_content) if is_nl_end
-                                                         else max(0, len(new_content) - 1)),
-                                   (0 if is_nl_end else self.end_lincol[1]) +
-                                   len(new_content[-1]) if len(new_content) != 0 else 0)
+                self.end_lincol = (self.end_lincol[0] + len(new_content) -
+                                   bool(not is_nl_end or is_empty), len(self.content[-1]))
 
-            self.content.extend(new_content[1:])
         return self
 
 
@@ -301,51 +300,61 @@ class Fragment():
         return new
 
 
-    def slice_match_obj(self, match_obj, groupno, after_inner=False, output_zero=True, **_):
+    def slice_match(self, match_obj, groupno, after_inner=False, output_zero=True, **_):
         """Slice by span defined by a regex match object."""
         if len(match_obj.groups()) >= groupno and match_obj.group(groupno) is not None:
-            at_start = self.loc_to_abs(match_obj.start(groupno))
-            at_end = self.loc_to_abs(match_obj.end(groupno))
-
-            return self.slice(at_start, at_end, after_inner, output_zero)
+            return self.slice(self.loc_to_abs(match_obj.start(groupno)),
+                              self.loc_to_abs(match_obj.end(groupno)),
+                              after_inner, output_zero)
 
 
-    def slice(self, at_start, at_end=None, after_inner=False, output_zero=True):
+    def slice(self, start=None, end=None, after_inner=False, output_zero=True):
         """Cut span."""
+        if start is None:
+            if end is None:
+                if after_inner:
+                    return self.copy()
+                if output_zero:
+                    return self.copy().clear(True), self.copy(), self.copy().clear(False)
+                return None, self.copy(), None
+
+            start = self.get_start(isinstance(end, int))
+
         start_pos_abs = None
-        if isinstance(at_start, int):
-            start_pos_abs = at_start
-            at_start = self.pos_to_lincol(at_start, True)
-            if at_end is not None:
-                at_end = self.pos_to_lincol(at_end, True)
+        if isinstance(start, int):
+            start_pos_abs = start
+            start = self.pos_to_lincol(start, True)
+            if end is not None:
+                end = self.pos_to_lincol(end, True)
 
-        at_start_rel = self.loc_to_rel(at_start)
-        if at_end is not None:
-            at_end_rel = self.loc_to_rel(at_end)
+        start_rel = self.loc_to_rel(start)
+        if end is not None:
+            end_rel = self.loc_to_rel(end)
 
-        start_rel = (0, 0)
+        self_start_rel = (0, 0)
         if self.end_lincol is not None:
-            end_rel = self.loc_to_rel(self.end_lincol)
+            self_end_rel = self.loc_to_rel(self.end_lincol)
         else:
-            end_rel = self.loc_to_rel(self.pos_to_lincol(self.end_pos, True))
+            self_end_rel = self.loc_to_rel(self.pos_to_lincol(self.end_pos, True))
 
         cuts = []
         if not after_inner:
-            cuts.append((start_rel, at_start_rel, self.start_pos, self.start_lincol))
-        if at_end is None:
-            cuts.append((at_start_rel, end_rel, start_pos_abs, at_start))
+            cuts.append((self_start_rel, start_rel, self.start_pos, self.start_lincol))
+        if end is None:
+            cuts.append((start_rel, self_end_rel, start_pos_abs, start))
         else:
-            cuts.append((at_start_rel, at_end_rel, start_pos_abs, at_start))
+            cuts.append((start_rel, end_rel, start_pos_abs, start))
             if not after_inner:
-                cuts.append((at_end_rel, end_rel, None, at_end))
+                cuts.append((end_rel, self_end_rel, None, end))
 
+        start_lincol_abs = start
         result = []
         for start, end, pos_abs, lincol_abs in cuts:
-            if start <= start_rel and end <= start_rel:
+            if start <= self_start_rel and end <= self_start_rel:
                 new = self.copy().clear(True) if output_zero else None
-            elif end >= end_rel and start <= start_rel:
+            elif end >= self_end_rel and start <= self_start_rel:
                 new = self.copy()
-            elif start >= end_rel:
+            elif start >= self_end_rel:
                 new = self.copy().clear(False) if output_zero else None
             elif start == end and not output_zero:
                 new = None
@@ -368,7 +377,7 @@ class Fragment():
                     if start_pos_abs is not None:
                         pos_abs = start_pos_abs
                     else:
-                        pos_abs = self.lincol_to_pos(at_start, True)
+                        pos_abs = self.lincol_to_pos(start_lincol_abs, True)
 
                 pos_abs = max(pos_abs, self.start_pos)
                 if self.start_lincol:
@@ -384,14 +393,26 @@ class Fragment():
         return result[0] if len(result) == 1 else tuple(result)
 
 
-    def slice_block(self, at_start, at_end, after_inner=False, output_zero=True,
+    def slice_block(self, start=None, end=None, after_inner=False, output_zero=True,
                     include_before=False, include_after=False):
         """Returns a rectangular block defined by the start and end corners."""
-        if isinstance(at_start, int):
-            at_start = self.pos_to_lincol(at_start)
-        if isinstance(at_end, int):
-            at_end = self.pos_to_lincol(at_end)
-        is_diff_column = bool(at_start[1] != at_end[1])
+        if start is None and end is None:
+            if after_inner:
+                return self.copy()
+            elif not output_zero:
+                return None, self.copy(), None
+
+        if isinstance(start, int):
+            start = self.pos_to_lincol(start)
+        if isinstance(end, int):
+            end = self.pos_to_lincol(end)
+
+        if start is None:
+            start = self.loc_to_abs((0, end[1] if end is not None else 0))
+        if end is None:
+            end = self.loc_to_abs((max(len(self.content) - 1, 0), start[1]))
+
+        is_diff_column = bool(start[1] != end[1])
 
         cuts = [FragmentBundle()]
         if not after_inner:
@@ -399,11 +420,11 @@ class Fragment():
         if is_diff_column:
             cuts.append(FragmentBundle())
 
-        for line in self.slice(at_start if not include_before else (at_start[0], 0),
-                               at_end if not include_after else (at_end[0]+1, 0),
+        for line in self.slice(start if not include_before else (start[0], 0),
+                               end if not include_after else (end[0]+1, 0),
                                True).splitlines():
-            result = line.slice((line.start_lincol[0], at_start[1]),
-                                (line.start_lincol[0], at_end[1]) if is_diff_column else None,
+            result = line.slice((line.start_lincol[0], start[1]),
+                                (line.start_lincol[0], end[1]) if is_diff_column else None,
                                 after_inner, output_zero)
             for cut, piece in zip(cuts, result):
                 if piece is not None:
@@ -536,23 +557,39 @@ class Fragment():
     def sort(self, *_):
         return None
 
-    #--------------------
-    # Location
 
-    def add_offset(self, offset_pos=None, offset_lincol=None):
-        """Adds an offset to the location."""
-        if offset_pos:
-            self.start_pos += offset_pos
-            self.end_pos += offset_pos
-        if offset_lincol and self.start_lincol:
+    # -- Location ------------------------------------------------------------
+
+    def move(self, pos=None, lincol=None):
+        """Moves the location."""
+        if pos:
+            self.end_pos = pos + self.span_len(True)
+            self.start_pos = pos
+
+        if lincol and self.start_lincol:
             if self.start_lincol[0] == self.end_lincol[0]:
-                self.end_lincol = (self.end_lincol[0] + offset_lincol[0],
-                                   self.end_lincol[1] + offset_lincol[1])
+                self.end_lincol = lincol + self.span_len(False)
             else:
-                self.end_lincol = (self.end_lincol[0] + offset_lincol[0], self.end_lincol[1])
+                self.end_lincol = (lincol[0] + self.span_len(False)[0], self.end_lincol[1])
 
-            self.start_lincol = (self.start_lincol[0] + offset_lincol[0],
-                                 self.start_lincol[1] + offset_lincol[1])
+            self.start_lincol = lincol
+
+        return self
+
+
+    def add_offset(self, pos=None, lincol=None):
+        """Adds an offset to the location."""
+        if pos:
+            self.start_pos += pos
+            self.end_pos += pos
+
+        if lincol and self.start_lincol:
+            if self.start_lincol[0] == self.end_lincol[0]:
+                self.end_lincol = (self.end_lincol[0] + lincol[0], self.end_lincol[1] + lincol[1])
+            else:
+                self.end_lincol = (self.end_lincol[0] + lincol[0], self.end_lincol[1])
+
+            self.start_lincol = (self.start_lincol[0] + lincol[0], self.start_lincol[1] + lincol[1])
 
         return self
 
@@ -712,8 +749,8 @@ class Fragment():
     def is_complete(self, *_):
         return True
 
-    #--------------------
-    # Size
+
+    # -- Size ----------------------------------------------------------------
 
     def __len__(self):
         """Returns the content char length."""
@@ -752,8 +789,8 @@ class Fragment():
     def is_empty(self):
         return len(self.content) == 0
 
-    #--------------------
-    # Iterate, Compare & Convert
+
+    # -- Iterate, Compare & Convert ------------------------------------------
 
     def __iter__(self):
         yield self
@@ -913,8 +950,8 @@ class FragmentBundle():
     def get_end(self, pos_lincol):
         return self.end_pos if pos_lincol else self.end_lincol
 
-    #--------------------
-    # Filename
+
+    # -- Filename ------------------------------------------------------------
 
     def has_consistent_filenames(self):
         """All filenames are the same."""
@@ -925,8 +962,8 @@ class FragmentBundle():
             prev = piece
         return True
 
-    #--------------------
-    # Content
+
+    # -- Content -------------------------------------------------------------
 
     def extend(self, new_content, keep_end=False):
         if not self:
@@ -949,7 +986,7 @@ class FragmentBundle():
         if not other:
             return self
 
-        is_before = bool(not self or other.get_end(pos_lincol) < self.get_end(pos_lincol))
+        is_before = bool(not self or other.get_start(pos_lincol) < self.get_end(pos_lincol))
 
         if keep_end and self:
             end_cur = self.get_end(pos_lincol)
@@ -1119,44 +1156,46 @@ class FragmentBundle():
         return new
 
 
-    def slice_match_obj(self, match_obj, groupno, after_inner=False, output_zero=True,
+    def slice_match(self, match_obj, groupno, after_inner=False, output_zero=True,
                         filler=None, filler_mode=None):
         """relative to first."""
         if not self:
-            return self
+            return self.copy()
         if len(match_obj.groups()) >= groupno and match_obj.group(groupno) is not None:
-            at_start = self.loc_to_abs(match_obj.start(groupno), filler, filler_mode)
-            at_end = self.loc_to_abs(match_obj.end(groupno), filler, filler_mode)
-
-            return self.slice(at_start, at_end, after_inner, output_zero)
+            return self.slice(self.loc_to_abs(match_obj.start(groupno), filler, filler_mode),
+                              self.loc_to_abs(match_obj.end(groupno), filler, filler_mode),
+                              after_inner, output_zero)
 
 
-    def slice(self, at_start, at_end=None, after_inner=False, output_zero=True):
-        if not self:
-            if after_inner:
-                return None
-            if not at_end:
-                return None, None
-            return None, None, None
+    def slice(self, start=None, end=None, after_inner=False, output_zero=True):
+        if not self or start is None:
+            if not self or end is None:
+                if after_inner:
+                    return self.copy()
+                if output_zero:
+                    return self.copy().clear(True), self.copy(), self.copy().clear(False)
+                return None, self.copy(), None
 
-        if at_end is not None and at_end < at_start:
-            at_end = at_start
+            start = self.get_start(isinstance(end, int))
 
-        pos_lincol = bool(isinstance(at_start, int))
+        if end is not None and end < start:
+            end = start
+
+        pos_lincol = bool(isinstance(start, int))
         cuts = []
-        at_start_index = self.index_clip(at_start, False)
+        start_index = self.index_clip(start, False)
         if not after_inner:
             cuts.append(((0, True), self.bundle[0].get_start(pos_lincol),
-                         at_start_index, at_start))
-        if not at_end:
-            cuts.append((at_start_index, at_start,
+                         start_index, start))
+        if not end:
+            cuts.append((start_index, start,
                          (max(0, len(self.bundle) - 1), True),
                          self.bundle[-1].get_end(pos_lincol)))
         else:
-            at_end_index = self.index_clip(at_end, True)
-            cuts.append((at_start_index, at_start, at_end_index, at_end))
+            end_index = self.index_clip(end, True)
+            cuts.append((start_index, start, end_index, end))
             if not after_inner:
-                cuts.append((at_end_index, at_end,
+                cuts.append((end_index, end,
                              (max(0, len(self.bundle) - 1), True),
                              self.bundle[-1].get_end(pos_lincol)))
 
@@ -1189,12 +1228,12 @@ class FragmentBundle():
                         new.bundle.append(pieces_inner.copy())
                     else:
                         for piece in self.bundle[min(len(self.bundle), index_start[0] + 1):
-                                              index_end[0]]:
+                                                 index_end[0]]:
                             new.bundle.append(piece.copy())
                     if index_end[1]:
                         piece_last = self.bundle[index_end[0]].slice(self.bundle[index_end[0]]
-                                                                  .get_start(pos_lincol),
-                                                                  end, True)
+                                                                     .get_start(pos_lincol),
+                                                                     end, True)
                         if piece_last is not None and len(piece_last) != 0:
                             new.bundle.append(piece_last)
 
@@ -1203,22 +1242,34 @@ class FragmentBundle():
         return result[0] if len(result) == 1 else tuple(result)
 
 
-    def slice_block(self, at_start, at_end, after_inner=False, output_zero=True,
+    def slice_block(self, start=None, end=None, after_inner=False, output_zero=True,
                     include_before=False, include_after=False):
-        if isinstance(at_start, int):
-            at_start = self.pos_to_lincol(at_start)
-        if isinstance(at_end, int):
-            at_end = self.pos_to_lincol(at_end)
+        if not self or (start is None and end is None):
+            if after_inner:
+                return self.copy()
+            elif not output_zero:
+                return None, self.copy(), None
+
+        if isinstance(start, int):
+            start = self.pos_to_lincol(start)
+        if isinstance(end, int):
+            end = self.pos_to_lincol(end)
+
+        if start is None:
+            start = self.loc_to_abs((0, end[1] if end is not None else 0))
+        if end is None:
+            end = self.bundle[-1].loc_to_abs((sum(max(len(piece.content) - 1, 0)
+                                                  for piece in self.bundle), start[1]))
 
         cuts = [FragmentBundle()]
         if not after_inner:
             cuts.append(FragmentBundle())
-        if at_start[1] != at_end[1]:
+        if start[1] != end[1]:
             cuts.append(FragmentBundle())
         for index, piece in enumerate(self):
-            result = piece.slice_block(at_start, at_end, after_inner, output_zero,
-                                    include_before or index != 0,
-                                    include_after or index != len(self.bundle) - 1)
+            result = piece.slice_block(start, end, after_inner, output_zero,
+                                       include_before or index != 0,
+                                       include_after or index != len(self.bundle) - 1)
             for cut, bundle in zip(cuts, result):
                 if bundle:
                     cut.combine(bundle, merge=False)
@@ -1332,16 +1383,29 @@ class FragmentBundle():
                              (lambda piece: (piece.get_end(pos_lincol),
                                              piece.get_start(pos_lincol))))
 
-    #--------------------
-    # Location
 
-    def add_offset(self, offset_pos=None, offset_lincol=None):
+    # -- Location ------------------------------------------------------------
+
+    def move(self, pos=None, lincol=None):
+        for piece in self:
+            lincol_piece = None
+            if lincol:
+                if piece.start_lincol[0] == self.start_lincol[0]:
+                    lincol_piece = lincol[0] + self.loc_to_rel(piece.start_lincol)
+                else:
+                    lincol_piece = (lincol[0] + self.loc_to_rel(piece.start_lincol)[0],
+                                    piece.start_lincol[1])
+            piece.move(pos + self.loc_to_rel(piece.start_pos), lincol_piece)
+        return self
+
+
+    def add_offset(self, pos=None, lincol=None):
         first_line = True
         for piece in self:
-            if offset_lincol and first_line and piece.start_lincol[0] != self.start_lincol[0]:
-                offset_lincol = (offset_lincol[0], 0)
+            if lincol and first_line and piece.start_lincol[0] != self.start_lincol[0]:
+                lincol = (lincol[0], 0)
                 first_line = False
-            piece.add_offset(offset_pos, offset_lincol)
+            piece.add_offset(pos, lincol)
         return self
 
 
@@ -1551,8 +1615,8 @@ class FragmentBundle():
             prev = piece
         return True
 
-    #--------------------
-    # Size
+
+    # -- Size ----------------------------------------------------------------
 
     def __len__(self):
         if not self:
@@ -1614,8 +1678,8 @@ class FragmentBundle():
                 return False
         return True
 
-    #--------------------
-    # Iterate, Compare & Convert
+
+    # -- Iterate, Compare & Convert ------------------------------------------
 
     def __iter__(self):
         """Iterate over bundle list."""
