@@ -19,6 +19,14 @@ class Node:
         self.child_nodes = LinkedList(self)
 
 
+    def detach(self):
+        """Remove links of the node."""
+        self.prev = None
+        self.next = None
+        self.parent_node = None
+        return self
+
+
     def prev_leaf(self):
         """Return the previous leaf node."""
         node = self
@@ -53,6 +61,15 @@ class Node:
                 return node
 
 
+    def copy(self, linked=False):
+        """Create a copy the node."""
+        new = type(self)()
+        for prop in self.__dict__.keys():
+            if linked or prop not in self.__slots__.keys():
+                setattr(new, prop, getattr(self, prop))
+        return new
+
+
 class LinkedList:
     """Container for interconnected nodes."""
 
@@ -69,10 +86,8 @@ class LinkedList:
     def copy(self):
         """Create a copy."""
         new_list = type(self)(self.parent)
-        new_list._head = self._head
-        new_list._tail = self._tail
-        new_list._list_size = self._list_size
-
+        for node in self:
+            new_list.append(node.copy(linked=False))
         return new_list
 
 
