@@ -101,8 +101,7 @@ class Report():
         self.fix = fix
 
 
-    #--------------------
-    # Severity
+    # -- Severity ------------------------------------------------------------
 
     fatal = 'F'
     error = 'E'
@@ -171,8 +170,20 @@ class Report():
     }
 
 
-    #--------------------
-    # Message
+    def map_severity(thresholds, value, start=None):
+        """Convert a value to a severity with a list of thresholds."""
+        severities = tuple(reversed(Report.severities))
+        if not start or start not in severities:
+            start = 'I'
+        direction = severities[0] <= severities[-1]
+        for index, threshold in enumerate(thresholds, -1 + severities.index(start)):
+            if bool(value > threshold if direction else value < threshold):
+                return severities[min(max(index, 0), len(severities) - 1)]
+        return severities[min(max(len(thresholds) -1 + severities.index(start), 0),
+                              len(severities) - 1)]
+
+
+    # -- Message -------------------------------------------------------------
 
     quantity = MessageTemplate("{what} {?where} {?how}").substitute
     existing = MessageTemplate("{what} {?where}").substitute
@@ -211,7 +222,12 @@ class Report():
         )
         if number != 1:
             if not irregular_plural:
-                subject += "s"
+                for suffix in {"s", "sh", "ch", "x", "z"}:
+                    if subject.endswith(suffix):
+                        subject += "es"
+                        break
+                else:
+                    subject += "s"
             else:
                 subject = irregular_plural
 
@@ -232,21 +248,109 @@ class Report():
         return str(number) + " " + subject
 
 
-    #--------------------
-    # Fix
+    # -- Fix -----------------------------------------------------------------
 
-    fix_mark_map = {
-        "letter": "/a/",
+    fix_map = (
+        {"letter": "/_/",
+        "long": "/noautofix/",
+        "ascii": "_",
+        "icon": "\u2B1C\uFE0E",
+        "emoji": "\u2B1C\uFE0F",
+        "color": "\u2B1C\uFE0F",
+        "shape": "\u2B55\uFE0E"},
+
+        {"letter": "/a/",
         "long": "/autofixed/",
         "ascii": "@",
         "icon": "\u2714\uFE0E",
         "emoji": "\u2714\uFE0F",
         "color": "\u2705\uFE0F",
-        "shape": "\U0001F6A9\uFE0E",
-    }
+        "shape": "\U0001F6A9\uFE0E"})
 
-    #--------------------
 
+    # -- Line ----------------------------------------------------------------
+
+    def set_line_lineno(self, code, start_end=True):
+        """Extract a single line.
+        start_end -- use start or end as location.
+        """
+        lineno = self.output.get_start(False)[0] if start_end else self.output.get_end(False)[0]
+        self.line = code.slice((lineno, 0), (lineno+1, 0), True)
+        return self
+
+
+    def set_line_newline(self, code, n, start_end=True):
+        """Extract a line including lines around it.
+        n -- number of lines to include around the line (odds below).
+        start_end -- use start or end as location.
+        """
+        lineno = self.output.get_start(False)[0] if start_end else self.output.get_end(False)[0]
+        start = (lineno - ceil(n / 2), 0)
+        end = (lineno + (n // 2) + 1, 0)
+        self.line = code.slice(start, end, True)
+        return self
+
+
+    def set_line_punc(self, code, min_chars, margin):
+        """Extracts line limited by punctuation.
+        min_chars -- minimal amount of chars to extract on both sides.
+        margin -- length of the outer margin within to search for punctuation marks.
+        """
+        start = self.output.start_pos - min_chars - margin
+        end = self.output.end_pos + min_chars + margin
+        buf = code.slice(start, end, True)
+
+        margin_start, _, margin_end = buf.slice(start + margin, end - margin)
+
+        if margin_start and len(margin_start) != 0:
+            margin_start_str = str(margin_start)
+            start_m = None
+            for start_m in re.finditer(r"[.?!:,;]", margin_start_str):
+                pass
+            if start_m:
+                start = margin_start.loc_to_abs(start_m.end(0))
+
+        if margin_end and len(margin_end) != 0:
+            if end_m := re.search(r"[.?!:,;]", str(margin_end)):
+                end = margin_end.loc_to_abs(end_m.end(0))
+
+        self.line = buf.slice(start, end, True)
+        return self
+
+
+    def set_line_offset(self, code, offset, after_before=None):
+        """Extracts a lines limited by an offset.
+        offset -- span to include.
+        after_before -- offset after end, before start or half on both sides.
+        """
+        is_pos = isinstance(offset, int)
+        start = self.output.get_start(is_pos)
+        end = self.output.get_end(is_pos)
+        if after_before is None:
+            if is_pos:
+                diff = offset // 2
+                start -= diff
+                end += diff
+            else:
+                diff = tuple(map(lambda e: e // 2 if e != 0 else 0, offset))
+                start = tuple(a - b for a, b in zip(start, diff))
+                end = tuple(a + b for a, b in zip(end, diff))
+        elif after_before:
+            if is_pos:
+                end += offset
+            else:
+                end = tuple(a + b for a, b in zip(end, offset))
+        else:
+            if is_pos:
+                start -= offset
+            else:
+                start = tuple(a - b for a, b in zip(start, offset))
+
+        self.line = code.slice(start, end, True)
+        return self
+
+
+    # ------------------------------------------------------------------------
 
     def repr(self, options=None):
         def join_entries(entries, options, formatting):
@@ -311,6 +415,7 @@ class Report():
             "marker_both": "¤",
 
             "show_autofix": False,
+            "show_autofix_always": False,
             "autofix_display": "long",
             **options
         }
@@ -374,9 +479,11 @@ class Report():
                 entries["line"] = entries["line"][:options["line_limit"]]
                 entries["line"] += options["line_ellipsis"]
 
-        if options["show_autofix"] and self.fix is not None:
-            entries["fix"] = self.fix_mark_map.get(options["autofix_display"],
-                                                   self.fix_mark_map["long"])
+        if (options["show_autofix"] and
+                (self.fix is not None or options["show_autofix_always"])):
+            entries["fix"] = self.fix_map[self.fix is not None].get(
+                                 options["autofix_display"],
+                                 self.fix_map[self.fix is not None]["long"])
 
         return join_entries(entries, options, formatting)
 
@@ -393,87 +500,7 @@ class Report():
                           self.line.copy(), self.fix.copy())
 
 
-    #------------------------
-
-
-    def getline_lineno(code, output, start_end=True):
-        """Extract a single line.
-        start_end -- use start or end as location.
-        """
-        lineno = output.start(False)[0] if start_end else output.end(False)[0]
-        return code.slice((lineno, 0), (lineno+1, 0), True)
-
-
-    def getline_newline(code, output, n, start_end=True):
-        """Extract a line including lines around it.
-        n -- number of lines to include around the line (odds below).
-        start_end -- use start or end as location.
-        """
-        lineno = output.start(False)[0] if start_end else output.end(False)[0]
-        start = (lineno - ceil(n / 2), 0)
-        end = (lineno + (n // 2) + 1, 0)
-        return code.slice(start, end, True)
-
-
-    def getline_punc(code, output, min_chars, margin):
-        """Extracts line limited by punctuation.
-        min_chars -- minimal amount of chars to extract on both sides.
-        margin -- length of the outer margin within to search for punctuation marks.
-        """
-        start = output.start_pos - min_chars - margin
-        end = output.end_pos + min_chars + margin
-        buf = code.slice(start, end, True)
-
-        margin_start, _, margin_end = buf.slice(start + margin, end - margin)
-
-        if margin_start and len(margin_start) != 0:
-            margin_start_str = str(margin_start)
-            start_m = None
-            for start_m in re.finditer(r"[.?!:,;]", margin_start_str):
-                pass
-            if start_m:
-                start = margin_start.loc_to_abs(start_m.end(0))
-
-        if margin_end and len(margin_end) != 0:
-            if end_m := re.search(r"[.?!:,;]", str(margin_end)):
-                end = margin_end.loc_to_abs(end_m.end(0))
-
-        return buf.slice(start, end, True)
-
-
-    def getline_offset(code, output, offset, after_before=None):
-        """Extracts a lines limited by an offset.
-        offset -- span to include.
-        after_before -- offset after end, before start or half on both sides.
-        """
-        is_pos = isinstance(offset, int)
-        start = output.get_start(is_pos)
-        end = output.get_end(is_pos)
-        if after_before is None:
-            if is_pos:
-                diff = offset // 2
-                start -= diff
-                end += diff
-            else:
-                diff = tuple(map(lambda e: e // 2 if e != 0 else 0, offset))
-                start = tuple(a - b for a, b in zip(start, diff))
-                end = tuple(a + b for a, b in zip(end, diff))
-        elif after_before:
-            if is_pos:
-                end += offset
-            else:
-                end = tuple(a + b for a, b in zip(end, offset))
-        else:
-            if is_pos:
-                start -= offset
-            else:
-                start = tuple(a - b for a, b in zip(start, offset))
-
-        return code.slice(start, end, True)
-
-
-#------------------------
-
+# ----------------------------------------------------------------------------
 
 def options_overide(options=None):
     """Override the default print options."""
@@ -493,20 +520,32 @@ def options_overide(options=None):
     }
 
 
-def print_reports(reports, options=None):
+def print_reports(reports, options=None, filename_prev=None, is_final=True):
     """Print the reports in the command line."""
-    if reports is None:
-        return None
+    if not reports:
+        return filename_prev
+
+    options = options_overide(options)
+    if options["sort_key"]:
+        key = {
+            "severity": lambda report: (report.output.filename, report.severity),
+            "tool": lambda report: (report.output.filename, report.tool),
+            "start_pos": lambda report: (report.output.filename, report.output.start_pos),
+            "start_lincol": lambda report: (report.output.filename, report.output.start_lincol),
+            "autofix": lambda report: (report.output.filename, bool(report.fix is None)),
+        }
+        # fail silently
+        if options["sort_key"] in key:
+            reports.sort(key=key[options["sort_key"]])
 
-    options = options_overide()
-    filename_prev = None
     for report in reports:
         if report is None:
             continue
         filename_prev = print_report(report, options, filename_prev)
 
-    if options["show_summary"]:
+    if is_final and options["show_summary"]:
         reports_summary(reports, options)
+    return reports[-1].output.filename
 
 
 def print_report(report, options=None, filename_prev=None):
